/***************************************************************************
                              kstviewlabel.h
                             ----------------
    begin                : Apr 10 2004
    copyright            : (C) 2000 by cbn
                           (C) 2004 by The University of Toronto
    email                :
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KSTVIEWLABEL_H
#define KSTVIEWLABEL_H

#include "kstbackbuffer.h"
#include "kstborderedviewobject.h"
#include "kstscalar.h"
#include "labelparser.h"

#include <qguardedptr.h>

class KstViewLabelDialogI;

class KstViewLabel : public KstBorderedViewObject {
  Q_OBJECT
  Q_PROPERTY(QString text READ text WRITE setText)
  Q_PROPERTY(double rotation READ rotation WRITE setRotation)  
  Q_PROPERTY(QString font READ fontName WRITE setFontName)
  Q_PROPERTY(int dataPrecision READ dataPrecision WRITE setDataPrecision)
  Q_PROPERTY(int fontSize READ fontSize WRITE setFontSize)
  Q_PROPERTY(bool transparent READ transparent WRITE setTransparent)
  Q_PROPERTY(int horizJustify READ horizJustifyWrap WRITE setHorizJustifyWrap)
  Q_PROPERTY(int vertJustify READ vertJustifyWrap WRITE setVertJustifyWrap)
  Q_PROPERTY(bool autoResize READ autoResize WRITE setAutoResize)
  public:
    KstViewLabel(const QString& txt, KstLJustifyType justify = 0L, float rotation = 0.0);
    KstViewLabel(const QDomElement& e);
    virtual ~KstViewLabel();

    void setText(const QString& text);
    const QString& text() const;

    void setRotation(double rotation);
    double rotation() const;

    void setJustification(KstLJustifyType Justify);
    KstLJustifyType justification() const { return _justify; }
    
    // wraps for Q_PROPERTIES
    int horizJustifyWrap() const;
    // 0 = left, 1 = right, 2 = centre
    void setHorizJustifyWrap(int justify);
    int vertJustifyWrap() const;
    // 0 = top, 1 = bottom, 2 = centre
    void setVertJustifyWrap(int justify);

    int ascent() const;

    /** Interpret special characters, default = true */
    void setInterpreted(bool interpreted);
    bool interpreted() const;

    void setAutoResize(bool on);
    bool autoResize() const;

    void setFontName(const QString& fontName);
    const QString& fontName() const;

    void setFontSize(int size);
    int fontSize() const;

    virtual void save(QTextStream& ts, const QString& indent = QString::null);

    void setDoScalarReplacement(bool in_do);
    bool doScalarReplacement() const;

    void setDataPrecision(int prec);
    int dataPrecision() const;
    
    void setTransparent(bool transparent);
    bool transparent() const;

    virtual void paint(KstPaintType type, QPainter& p, const QRegion& bounds);
    virtual void resize(const QSize&);
    virtual QRegion clipRegion();
    
    virtual QMap<QString, QVariant> widgetHints(const QString& propertyName) const;

  public slots:
    void adjustSizeForText(QRect w);
    void reparse();

  protected:
    virtual KstViewObjectFactoryMethod factory() const;
    virtual bool layoutPopupMenu(KPopupMenu *menu, const QPoint& pos, KstViewObjectPtr topLevelParent);
    virtual void readBinary(QDataStream& str);
    virtual void writeBinary(QDataStream& str);

  private:
    void drawToBuffer(Label::Parsed *lp);
    void drawToPainter(Label::Parsed *lp, QPainter& p);
    void computeTextSize(Label::Parsed*);

    double _rotation;
    QString _txt;
    QString _fontName;
    KstScalarList _scalarsUsed;

    bool _replace : 1;
    bool _interpret : 1;
    bool _autoResize : 1;
    int _absFontSize; // points
    int _fontSize; // size relative to reference size.....
    int _dataPrecision : 6;
    int _textWidth, _textHeight, _ascent;
    KstLJustifyType _justify;
    KstBackBuffer _backBuffer;
    Label::Parsed *_parsed;
};

typedef KstSharedPtr<KstViewLabel> KstViewLabelPtr;

#endif
// vim: ts=2 sw=2 et
